"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Observable_1 = require("rxjs/Observable");
const Subject_1 = require("rxjs/Subject");
const stream = require("stream");
/**
 * Transform the output of an Observable into a node readable stream.
 */
function rxToStream(src, options = { encoding: 'utf8' }, onError) {
    const trigger = new Subject_1.Subject();
    let depth = 0;
    const maxDepth = 100;
    const readable = new stream.Readable(Object.assign({}, options, { read: () => {
            trigger.next();
        } }));
    function close(err) {
        try {
            if (err && onError)
                onError(err, readable);
        }
        finally {
            Observable_1.Observable.interval(1).take(1).subscribe(() => readable.push(null));
        }
    }
    function next() {
        if (depth < maxDepth) {
            // Use the fastest method
            trigger.next();
        }
        else {
            // Slower to avoid running out of stack space.
            Observable_1.Observable.interval().take(1).subscribe(() => trigger.next());
        }
    }
    function push(data) {
        depth += 1;
        readable.push(data) ? next() : null;
        depth -= 1;
    }
    trigger
        // Use zip to buffer the Observable so we only send when the stream is ready.
        .zip(src, (_, src) => src)
        .subscribe(
    // send the data and signal we can use more data.
    push, 
    // Close on error or complete.
    close, close);
    return readable;
}
exports.rxToStream = rxToStream;
//# sourceMappingURL=rxToStream.js.map